program CloudPlasma;

uses Crt;

var
  p, q : Word;

{**************************************************************************
** Set to VGA 320x200 256 colours                                        **
**************************************************************************}
procedure VGAScreen; assembler;
asm
  mov ax, $13
  int $10
end;

{**************************************************************************
** Set to text mode                                                      **
**************************************************************************}
procedure TextScreen; assembler;
asm
  mov ax, $3
  int $10
end;

{**************************************************************************
** Sets a DAC register to a specific Red Green Blue-value                **
**************************************************************************}
procedure SetDAC (DAC, R, G, B : Byte);
begin
  Port[$3c8] := DAC;
  Port[$3c9] := R;
  Port[$3c9] := G;
  Port[$3c9] := B;
end;

{**************************************************************************
** Sets up a smooth palette ranging from blue to white                   **
** This leaves colour 0 untouched                                        **
**************************************************************************}
procedure SmoothPalette;
var
  Value : Byte;

begin
  for Value := 1 to 255 do
    SetDAC(Value,
           Value shr 2, {Red fades in from none to full}
           Value shr 2, {Green fades in from none to full}
           63);         {Blue is always at full strength}
end;

{**************************************************************************
** Sets a single pixel on the screen to a specified colour               **
**************************************************************************}
procedure PutPixel (x, y : Word; c : Byte);
begin
  Mem [$A000 : y shl 8 + y shl 6 + x] := c;
end;

{**************************************************************************
** Reads the colour of a specified pixel on the screen                   **
**************************************************************************}
function ReadPixel (x, y : Word) : Byte;
begin
  ReadPixel := Mem [$A000 : y shl 8 + y shl 6 + x];
end;


procedure Plasma (x1, y1, x2, y2 : Word);
var
  TopLeft, TopRight, BottomLeft, BottomRight : Byte;
  Left, Right, Top, Bottom, Center : Byte;
begin
  {Make sure recursion ends at a point}
  if (x2 - x1 < 2) and (y2 - y1 < 2) then exit;
  {Read colours}
  TopLeft     := ReadPixel (x1, y1);
  TopRight    := ReadPixel (x2, y1);
  BottomLeft  := ReadPixel (x1, y2);
  BottomRight := ReadPixel (x2, y2);
  {Average colours}
  Left   := (TopLeft + BottomLeft) shr 1;
  Right  := (TopRight + BottomRight) shr 1;
  Top    := (TopLeft + TopRight) shr 1;
  Bottom := (BottomLeft + BottomRight) shr 1;
  Center := (TopLeft + TopRight + BottomLeft + BottomRight) shr 2;
  {Define four quadrants}
  PutPixel (x1, (y1 + y2) shr 1, Left);
  PutPixel (x2, (y1 + y2) shr 1, Right);
  PutPixel ((x1 + x2) shr 1, y1, Top);
  PutPixel ((x1 + x2) shr 1, y2, Bottom);
  PutPixel ((x1 + x2) shr 1, (y1 + y2) shr 1, Center);
  {Make a smaller plasma in each of the four quadrants}
  Plasma (x1, y1, (x1 + x2) shr 1, (y1 + y2) shr 1); {top left}
  Plasma ((x1 + x2) shr 1, (y1 + y2) shr 1, x2, y2); {bottom right}
  Plasma ((x1 + x2) shr 1, y1, x2, (y1 + y2) shr 1); {top right}
  Plasma (x1, (y1 + y2) shr 1, (x1 + x2) shr 1, y2); {bottom left}
  {Ah! Recursion}
end;

begin
  {Clear Screen}
  ClrScr;

  {Display message}
  Writeln ('PLASMA Demonstration by Steven Don');
  Writeln ('----------------------------------');
  Writeln ('This will generate a BIG plasma cloud.');
  Writeln ('It has only four different colours, one at each corner.');
  Writeln ('Because of this, it looks far too smooth.');
  Writeln;
  Writeln ('Press any key to see what I mean.');
  {Wait for a keypress}
  ReadKey;

  {Switch to VGA mode}
  VGAScreen;

  {Set up palette to shades of blue}
  SmoothPalette;

  {Make sure we get a different plasma cloud each time}
  Randomize;

  {Create one large plasma, too smooth, but you can see how it works}
  PutPixel (  0,   0, Random (254) + 1);
  PutPixel (319,   0, Random (254) + 1);
  PutPixel (  0, 199, Random (254) + 1);
  PutPixel (319, 199, Random (254) + 1);
  Plasma (0, 0, 319, 199);
  {Wait for a keypress}
  ReadKey;

  {Switch back to text}
  TextScreen;

  {Display message}
  Writeln ('PLASMA Demonstration by Steven Don');
  Writeln ('----------------------------------');
  Writeln ('However, if you''d make the big one a bit smaller and combined more of them,');
  Writeln ('it would look a lot better.');
  Writeln;
  Writeln ('Press any key to see the result.');
  {Wait for a keypress}
  ReadKey;

  {Switch back to VGA}
  VGAScreen;
  {Palette was changed when we switched to text, so set it again}
  SmoothPalette;

  {Create 16 small plasma's, much better}
  for p := 0 to 3 do begin
    {Set top left pixels}
    PutPixel ( 80 + p * 40, 20, Random (254) + 1);
    PutPixel ( 80, 20 + p * 40, Random (254) + 1);
  end;
  for q := 0 to 3 do
    for p := 0 to 3 do begin
      {Set bottom right pixel}
      PutPixel (120 + p * 40, 60 + q * 40, Random (254) + 1);
      {Generate small plasma}
      Plasma ( 80 + p * 40, 20 + q * 40, 120 + p * 40, 60 + q * 40);
    end;

  {Wait for a keypress}
  Readkey;

  {Switch back to text}
  TextScreen;
end.